/* biblio.c */

/* Direttive */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "listalibri.h"


/* Prototipi delle procedure secondarie */

void InterpretaLineaComando (int argc, char *argv[], char *file_libri, char *file_prestiti);

/* Carica un elenco di libri dal file file_libri */
listalibri CaricaLibri (char *file_libri);

/* Stampa l'elenco dei libri disponibili */
void StampaLibri (listalibri L);

/* Esegue i movimenti riportati nel file dei prestiti e dei resi
   sulle liste Scaffale, Prestiti e Resi */
void EsegueMovimenti (char *file_prestiti, listalibri *pScaffale,
                      listalibri *pPrestiti, listalibri *pResi);

/* Stampa ed esegue le istruzioni per riporre i libri resi nello scaffale */
void RiordinaResi (listalibri *pResi, listalibri *pScaffale);


/* Programma principale */
int main (int argc, char *argv[])
{
  /* Parte dichiarativa */
  char file_libri[ROW_LENGTH];
  char file_prestiti[ROW_LENGTH];
  listalibri Scaffale;
  listalibri Prestiti;
  listalibri Resi;

  /* Parte esecutiva */
  InterpretaLineaComando(argc,argv,file_libri,file_prestiti);

  /* Carica l'elenco dei libri disponibili */
  Scaffale = CaricaLibri(file_libri);

  /* Crea le liste (vuote) dei libri prestati e resi */
  Prestiti = crealista();
  Resi = crealista();

  /* Esegue i movimenti riportati nel file dei prestiti e dei resi */
  EsegueMovimenti(file_prestiti,&Scaffale,&Prestiti,&Resi);

  /* Stampa l'elenco dei libri disponibili */
  StampaLibri(Scaffale);

  /* Stampa ed esegue le istruzioni per riporre i libri resi */
  RiordinaResi(&Resi,&Scaffale);

  /* Stampa l'elenco dei libri disponibili */
  StampaLibri(Scaffale);

  /* Dealloca le strutture dinamiche */
  distruggelista(&Scaffale);
  distruggelista(&Prestiti);
  distruggelista(&Resi);

  return EXIT_SUCCESS;
}


/* Definizione delle procedure secondarie */

void InterpretaLineaComando (int argc, char *argv[], char *file_libri, char *file_prestiti)
{
  if (argc != 3)
  {
    fprintf(stderr,"Errore nella linea di comando!\n");
    exit(EXIT_FAILURE);
  }

  strcpy(file_libri,argv[1]);
  strcpy(file_prestiti,argv[2]);
}


/* Carica un elenco di libri dal file file_libri */
listalibri CaricaLibri (char *file_libri)
{
  FILE *fp;
  listalibri Scaffale;
  char Riga[ROW_LENGTH];
  posizione p;

  /* Apre il file dei libri */
  fp = fopen(file_libri,"r");
  if (fp == NULL)
  {
    fprintf(stderr,"Errore nell'apertura del file %s!\n",file_libri);
    exit(EXIT_FAILURE);
  }

  /* Crea la lista Scaffale */
  Scaffale = crealista();

  /* Per ogni riga del file */
  while (fscanf(fp,"%[^\n]\n",Riga) == 1)
  {
    /* Se e' il comando di fine, esce dal ciclo */
    if (strcmp(Riga,"FINE") == 0) break;

    /* Se e' un libro, lo aggiunge alla lista Scaffale DOPO l'ultimo elemento */
    p = succlista(Scaffale,ultimolista(Scaffale));
    Scaffale = inslista(Scaffale,p,Riga);
  }

  /* Chiude il file dei libri */
  fclose(fp);

  return Scaffale;
}


/* Stampa l'elenco dei libri disponibili */
void StampaLibri (listalibri L)
{
  posizione p;
  libro AutoreTitolo;

  for (p = primolista(L); !finelista(L,p); p = succlista(L,p))
  {
    leggelista(L,p,AutoreTitolo);
    printf("%s\n",AutoreTitolo);
  }
  printf("FINE\n");
}


/* Esegue i movimenti riportati nel file dei prestiti e dei resi
   sulle liste Scaffale, Prestiti e Resi */
void EsegueMovimenti (char *file_prestiti, listalibri *pScaffale,
                      listalibri *pPrestiti, listalibri *pResi)
{
}


/* Stampa ed esegue le istruzioni per riporre i libri resi nello scaffale */
void RiordinaResi (listalibri *pResi, listalibri *pScaffale)
{
}
